/* nouveau_drv.c.c -- nouveau nouveau driver -*- linux-c -*-
 * Created: Wed Feb 14 17:10:04 2001 by gareth@valinux.com
 */
/*-
 * Copyright 2000 VA Linux Systems, Inc., Sunnyvale, California.
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * VA LINUX SYSTEMS AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Authors:
 *    Gareth Hughes <gareth@valinux.com>
 *
 */

#include "drmP.h"
#include "drm.h"
#include "nouveau_drv.h"
#include "drm_pciids.h"

extern struct drm_ioctl_desc nouveau_ioctls[];
extern int nouveau_max_ioctl;

/* drv_PCI_IDs for nouveau is just to match the vendor id */
static drm_pci_id_list_t nouveau_pciidlist[] = {
	nouveau_PCI_IDS
};

static void 
nouveau_configure(struct drm_device *dev)
{
	dev->driver->driver_features =
	   DRIVER_USE_AGP | DRIVER_PCI_DMA | DRIVER_SG | DRIVER_HAVE_IRQ;

	dev->driver->buf_priv_size	= sizeof(struct drm_nouveau_private);
	dev->driver->load		= nouveau_load;
	dev->driver->unload		= nouveau_unload;
	dev->driver->firstopen		= nouveau_firstopen;
	dev->driver->preclose		= nouveau_preclose;
	dev->driver->lastclose		= nouveau_lastclose;
	dev->driver->irq_preinstall	= nouveau_irq_preinstall;
	dev->driver->irq_postinstall	= nouveau_irq_postinstall;
	dev->driver->irq_uninstall	= nouveau_irq_uninstall;
	dev->driver->irq_handler	= nouveau_irq_handler;

	dev->driver->ioctls		= nouveau_ioctls;
	dev->driver->max_ioctl		= nouveau_max_ioctl;

	dev->driver->name		= DRIVER_NAME;
	dev->driver->desc		= DRIVER_DESC;
	dev->driver->date		= DRIVER_DATE;
	dev->driver->major		= DRIVER_MAJOR;
	dev->driver->minor		= DRIVER_MINOR;
	dev->driver->patchlevel		= DRIVER_PATCHLEVEL;
}

static int
nouveau_probe(device_t kdev, cfdata_t match, void *aux)
{
	struct pci_attach_args *pa = aux;
	
	return drm_probe(pa, nouveau_pciidlist);
}

static bool
nv_suspend(device_t self PMF_FN_ARGS) 
{
    struct drm_device *dev = device_private(self);
    if(nouveau_suspend(dev) == 0)
	return true;
    else
	return false;
}

static bool
nv_resume(device_t self PMF_FN_ARGS)
{
    struct drm_device *dev = device_private(self);
    if(nouveau_resume(dev) == 0)
	return true;
    else
	return false;
}

static void
nouveau_attach(device_t kdev, device_t self, void *aux)
{

	struct pci_attach_args *pa = aux;
	struct drm_device *dev = device_private(self);

	dev->driver = malloc(sizeof(struct drm_driver_info), DRM_MEM_DRIVER,
	    M_WAITOK | M_ZERO);

	nouveau_configure(dev);
	pmf_device_register(self, nv_suspend, nv_resume);
	drm_attach(self, pa, nouveau_pciidlist);
}


CFATTACH_DECL_NEW(nouveau, sizeof(struct drm_device), nouveau_probe, nouveau_attach, 
drm_detach, drm_activate);

#ifdef _MODULE

MODULE(MODULE_CLASS_DRIVER, nouveau, "nouveau");

CFDRIVER_DEC(nouveau, DV_NULL, NULL);
extern struct cfattach nouveau_ca;
static int drmloc[] = { -1 };
static struct cfparent drmparent = {
    "drm", "vga", DVUNIT_ANY
};

static struct cfdata nouveau_cfdata[] = {
    {
	.cf_name = "nouveau",
	.cf_atname = "nouveau",
	.cf_unit = 0,
	.cf_fstate = FSTATE_STAR,
	.cf_loc = drmloc,
	.cf.flags = 0,
	.cf.pspec = &drmparent,
    },
    { NULL }
};

static int
nouveau_modcmd(modcmd_t cmd, void *arg)
{
    int err;
    
    switch (cmd) {
    case MODULE_CMD_INIT:
	err = config_cfdriver_attach(&nouveau_cd);
	if (err)
	    return err;
	err = config_cfattach_attach("nouveau", &nouveau_ca);
	if (err) {
	    config_cfdriver_detach(&nouveau_cd);
	    return err;
	}
	err = config_cfdata_detach(nouveaudrm_cfdata, 1);
	if (err) {
	    config_cfattach_detach("nouveau", &nouveau_ca);
	    config_cfdriver_detach(&nouveau_cd);
	    return err;
	}
	return 0;
	case MODULE_CMD_FINI:
	    err = config_cfdata_detach(nouveau_cfdata);
	    if (err)
		return err;
	    config_cfattach_detach("nouveau", &nouveau_ca);
	    config_cfdriver_detach(&nouveau_cd);
	    return 0;	
	default:
	    return ENOTTY;
    }
}
#endif /* _MODULE */

